import yaml
import sqlite3
import tibis.lib.args_parser as args_parser
import tibis.lib.logger as log
import tibis.lib.static as static
import tibis.lib.first_time as firstTime
from pathlib import Path

def main():
    path=Path(static.tibis_full_config_location)
    if(not path.is_file()):
        firstTime.firstTime()
    #log.info("Setup database if not exists")
    conn=None
    try:
        conn=sqlite3.connect(static.tibis_db_location)
        cursor_obj = conn.cursor()
        table = static.tibis_database_init;
        cursor_obj.execute(table)
    except sqlite3.Error as e:
        raise e
    finally:
        if conn:
            conn.close()


    args_parser.args_parser()

if __name__ == "__main__":
 main()



from pathlib import Path

version  = '1.0.7'
codename = "Ibis is born"

tibis_config_file="config.yml"
tibis_config_location=str(Path.home())+"/.config/tibis/"
tibis_full_config_location=tibis_config_location+tibis_config_file

tibis_keys_location=tibis_config_location+"keys"


tibis_dbfile="database.db"
tibis_db_location=tibis_config_location+"/"+tibis_dbfile
tibis_database_init=""" CREATE TABLE IF NOT EXISTS tibis (
            name VARCHAR(255) NOT NULL,
            private_key_path VARCHAR(255) NOT NULL,
            public_key_path VARCHAR(255) NOT NULL,
            status VARCHAR(10) NOT NULL,
            mount_point VARCHAR(255) NOT NULL
        ); """

tibis_database_insert=""" INSERT INTO tibis(name,private_key_path,public_key_path,status,mount_point) VALUES(?,?,?,?,?) """
tibis_database_list_all="""SELECT name,private_key_path,public_key_path,status,mount_point FROM tibis"""

tibis_empty_dir="/tmp/tempate_tibis_empty_dir"
tibis_storage_path="/tmp/storage"
tibis_tmp_dir="/tmp/tibis_tmp"

default_config={'storage_path':tibis_storage_path,'keys_location':tibis_keys_location,'pgp_infos':{'email':'tibis@localhost.org','comment':'Tibis Key','name':'Tibis'},'check_integrity':True}




import pgpy
import getpass
import tibis.lib.common as common
import tibis.lib.config as config
import tibis.lib.static as static
import tibis.lib.logger as log
from pathlib import Path
from tibis.lib.common import getPrivateKey

def unlock(dirname,destination):
 #Verifier que c'est pas déja unlock
 if(not common.existsInDB(dirname)):
  log.error("Directory not in database")
  return False
 if(common.isUnlocked(dirname)):
  log.error("Already unlocked")
  return False
 else:
  #GetKey
  enc_privatekey, _ = pgpy.PGPKey.from_file(static.tibis_keys_location+"/"+dirname+"/private")
  #Unlock key
  log.ask("Enter passphrase :")
  p = getpass.getpass("")

  try:
    with enc_privatekey.unlock(p) as privatekey:
        #Uncrypt datas
        encrypted_content=pgpy.PGPMessage.from_file(config.storage_path()+"/"+dirname)
        plaintext=privatekey.decrypt(encrypted_content).message
        #TODO Verify if its ok
        #Empty tmp dir
        [f.unlink() for f in Path(static.tibis_tmp_dir).glob("*") if f.is_file()]
        with open(static.tibis_tmp_dir+"/"+dirname,'wb') as archive:
            archive.write(plaintext)

        #Prepare output
        path=Path(destination)
        path.mkdir(exist_ok=True,parents=True)
        common.uncompressArchive(static.tibis_tmp_dir+"/"+dirname,destination)
        if(common.updateStatus(dirname,"unlocked") and common.updateMountPoint(dirname,destination)):
            log.success("Open at "+destination)
        [f.unlink() for f in Path(static.tibis_tmp_dir).glob("*") if f.is_file()]
  except Exception as e: 
    print(e)

if __name__ == '__main__':
    unlock(dirname,destination)



import sqlite3
import tibis.lib.logger as log
import tibis.lib.static as static
import tibis.lib.common as common 

import simple_chalk as chalk

def delete(dirname):
 if(not common.existsInDB(dirname)):
  log.error("Entry does not exist")
 else:
   answer = ""
   while answer not in ["y", "n"]:
    answer = input(chalk.red.bold("OK to push to continue [Y/N]? ")).lower()
   if(answer=="y"):
    #On vire la data  du storage
    if(common.isUnlocked(dirname)):
      mp=common.getMountPoint(dirname)
      log.warning("Your content was previously unlocked at "+ mp +" you have to delete it by yourself")
      try:
       common.remove_dir(static.tibis_keys_location+"/"+dirname)
      except:
       log.warning("Keys are already gone")
      common.deleteSQLEntry(dirname)
      log.warning("Entry and keys removed")
   else:
    log.success("Leaves the things as it ...")     

if __name__ == '__main__':
  deleteSQLEntry(dirname)


import tarfile
import sqlite3
import hashlib
import pgpy
import tibis.lib.logger as log
import tibis.lib.static as static
import tibis.lib.config as config

from pathlib import Path
import os 
import sys
import threading
import time

from pyspin.spin import Spin5, Spinner

runningSpinner = True 

def existsInDB(name):
	rows=True
	try:
		db=static.tibis_db_location
		conn=sqlite3.connect(db)
		cursor_obj = conn.cursor()
		cursor_obj.execute("SELECT name FROM tibis WHERE name=?", [name])

		rows = cursor_obj.fetchall()
	except Exception as e:
		raise e
	finally:
		conn.close()
		if(rows):
			return True
		else:
			return False

def getPrivateKey(name):
	rows=True
	try:
		db=static.tibis_db_location
		conn=sqlite3.connect(db)
		cursor_obj = conn.cursor()
		cursor_obj.execute("SELECT private_key_path as private FROM tibis WHERE name=?", [name])
		rows = cursor_obj.fetchone()	
	except Exception as e:
		raise e
	finally:
		conn.close()
		private_key_path=rows[0]
		if(Path(private_key_path).is_file()):
			return private_key_path
		else:
			return False

def getPublicKey(name):
	rows=True
	try:
		db=static.tibis_db_location
		conn=sqlite3.connect(db)
		cursor_obj = conn.cursor()
		cursor_obj.execute("SELECT public_key_path as public FROM tibis WHERE name=?", [name])
		rows = cursor_obj.fetchone()	
	except Exception as e:
		raise e
	finally:
		conn.close()
		public_key_path=rows[0]
		if(Path(public_key_path).is_file()):
			return public_key_path
		else:
			return False

def getMountPoint(name):
	rows=True
	try:
		db=static.tibis_db_location
		conn=sqlite3.connect(db)
		cursor_obj = conn.cursor()
		cursor_obj.execute("SELECT mount_point as mp FROM tibis WHERE name=?", [name])
		rows = cursor_obj.fetchone()	
	except Exception as e:
		raise e
	finally:
		conn.close()
		mp=rows[0]
		if(Path(mp).is_dir()):
			return mp
		else:
			return False

def updateMountPoint(name,mountPoint):
	allGood=False
	if(existsInDB(name)):
		try:
			db=static.tibis_db_location
			conn=sqlite3.connect(db)
			cursor_obj=conn.cursor()
			cursor_obj.execute("UPDATE tibis SET mount_point=? WHERE name=?",[mountPoint,name])
			conn.commit()
			allGood=True
		except Exception as e:
			raise e 
		finally:
			conn.close()
			return allGood

def isUnlocked(name):
	rows=True
	try:
		db=static.tibis_db_location
		conn=sqlite3.connect(db)
		cursor_obj = conn.cursor()
		cursor_obj.execute("SELECT status as status FROM tibis WHERE name=?", [name])
		rows = cursor_obj.fetchone()	
	except Exception as e:
		raise e
	finally:
		conn.close()
		status=rows[0]
		if(status=='unlocked'):
			return True
		else:
			return False

def updateStatus(name,status):
	allGood=False
	if(existsInDB(name)):
		try:
			db=static.tibis_db_location
			conn=sqlite3.connect(db)
			cursor_obj=conn.cursor()
			cursor_obj.execute("UPDATE tibis SET status=? WHERE name=?",[status,name])
			conn.commit()
			allGood=True
		except Exception as e:
			raise e 
		finally:
			conn.close()
			return allGood

def uncompressArchive(source,dest):
 with tarfile.open(source,"r:gz") as tar:
  def is_within_directory(directory, target):
      
      abs_directory = os.path.abspath(directory)
      abs_target = os.path.abspath(target)
  
      prefix = os.path.commonprefix([abs_directory, abs_target])
      
      return prefix == abs_directory
  
  def safe_extract(tar, path=".", members=None, *, numeric_owner=False):
  
      for member in tar.getmembers():
          member_path = os.path.join(path, member.name)
          if not is_within_directory(path, member_path):
              raise Exception("Attempted Path Traversal in Tar File")
  
      tar.extractall(path, members, numeric_owner=numeric_owner) 
      
  
  safe_extract(tar, dest)

def deleteArchive(source):
	remove_dir(source)

def createArchive(dirname,source,dest):
	with tarfile.open(dest+"/"+dirname+".gz","w:gz") as tar:
		for fn in os.listdir(source):
			p = os.path.join(source, fn)
			tar.add(p, arcname=fn)
		#tar.add(source,arcname=dirname)
	return dest+"/"+dirname+".gz"

def cryptArchive(keyPath,source,dest,dirname):
	try:
		pubkey,_ = pgpy.PGPKey.from_file(keyPath)
		# file = open(source, "rb")
		# data = file.read()
		# file.close()
		# message = pgpy.PGPMessage.new(data)
		file_message=pgpy.PGPMessage.new(source,file=True)
		encrypted_message = pubkey.encrypt(file_message)

		#Important remove the clear content
		deleteArchive(source)
		#Save data into storage
		outputfile=dest+"/"+dirname

		#bytes_data=bytes(encrypted_message)

		with open(outputfile,'wb') as destFile:
			destFile.write(bytes(encrypted_message))
		return True
	except Exception as e:
		print(e)
		return False

def remove_dir(directory):
    path=Path(directory)
    if path.is_file() or path.is_symlink():
        path.unlink()
        return
    for p in path.iterdir():
        remove_dir(p)
    path.rmdir()

def deleteSQLEntry(name):
	allGood=False
	if(existsInDB(name)):
		try:
			db=static.tibis_db_location
			conn=sqlite3.connect(db)
			cursor_obj=conn.cursor()
			cursor_obj.execute("DELETE FROM tibis WHERE name=?",[name])
			conn.commit()
			allGood=True
		except Exception as e:
			raise e 
		finally:
			conn.close()
			return allGood

def calculate_hash(file_content, algorithm="sha256"):
    hasher = hashlib.new(algorithm)
    hasher.update(file_content)
    return hasher.hexdigest()

def calculate_tar_hash(archive_path, algorithm="sha256"):
    tar = tarfile.open(archive_path, "r")
    file_hashes = {}

    for member in tar.getmembers():
        if member.isfile():
            file_content = tar.extractfile(member).read()
            file_hash = calculate_hash(file_content, algorithm)
            file_hashes[member.name] = file_hash

    tar.close()
    return file_hashes

def calculate_directory_hash(directory,algorithm="sha256"):
    file_hashes = {}

    for root, dirs, files in os.walk(directory):
        for file in files:
            file_path = os.path.join(root, file)
            with open(file_path, 'rb') as f:
                content = f.read()
                #file_hash = hashlib.sha256(content).hexdigest()
                file_hash = calculate_hash(content, algorithm)
                file_hashes[file_path] = file_hash
    return file_hashes 

def checkArchiveIntegrity(archive_path):

	global runningSpinner
	# Create a separate thread for the spinning cursor
	spinner_thread = threading.Thread(target=spinning_cursor)
	spinner_thread.start()

	value = calculate_tar_hash(archive_path)
	runningSpinner = False
	# Wait for the spinner thread to finish
	spinner_thread.join(0)
	sys.stdout.flush()
	
	return value

def spinning_cursor():
	spin = Spinner(Spin5)
	global runningSpinner
	while runningSpinner:
		for i in range(50):
			print(u"\r{0} Checking Integrity ...".format(spin.next()), end="")
			sys.stdout.flush()
			time.sleep(0.1)

def checkIntegrityIsOK(archiveIntegrity,directoryIntegrity,mountPoint):
	if(len(archiveIntegrity) != len(directoryIntegrity)):
		log.error("Not the same files")
		log.error("Archive Content : "+archiveIntegrity)
		log.error("Directory Content : "+directoryIntegrity)
		sys.exit("ERROR")

	_directoryIntegrity=[]
	_archiveIntegrity=[]

	if(mountPoint[::-1][0]!='/'):
		mountPoint+="/"
	#CleanDirectoryIntegrity to remove mountPointValue
	for obj in directoryIntegrity:
		_directoryIntegrity.append({obj.replace(mountPoint,""):directoryIntegrity[obj]})
	for obj in archiveIntegrity:
		_archiveIntegrity.append({obj:archiveIntegrity[obj]})
	
	# Convert data1 and data2 to sets of frozensets
	set1 = {frozenset(item.items()) for item in _archiveIntegrity}
	set2 = {frozenset(item.items()) for item in _directoryIntegrity}

	if(set1!=set2 or set2!=set1):
		log.error("Integrity error")
		log.error("Archive Content : "+archiveIntegrity)
		log.error("Directory Content : "+directoryIntegrity)
		sys.exit(1)
	else:
		log.success("Same integrity between archive and content")


import simple_chalk as chalk

def info(text):
 toPrint="[+] " + text
 print(chalk.blue.bold(toPrint))

def success(text):
 toPrint="[V] "+ text 
 print(chalk.green.bold(toPrint))

def ask(text):
 toPrint="[?] "+ text
 print(chalk.yellow.bold(toPrint))

def warning(text):
 toPrint="[!] "+ text
 print(chalk.yellow.bold(toPrint))

def error(text):
 toPrint="[?] "+ text
 print(chalk.red.bold(toPrint))

def locked(text):
 toPrint="[X] "+ text
 print(chalk.green(toPrint))

def unlocked(text):
 toPrint="[O] "+ text
 print(chalk.red(toPrint))


import yaml 
import simple_chalk as chalk
import tibis.lib.logger as log
import tibis.lib.static as static

from pathlib import Path


def firstTime():
 log.info("Create config file")
 path=Path(static.tibis_config_location)
 path.mkdir(parents=True,exist_ok=True)
 log.success("Done")

 log.info("Create key directory")
 path=Path(static.tibis_keys_location)
 path.mkdir(parents=True,exist_ok=True)
 log.success("Done")

 log.ask("Storage directory :")
 storage_path=input("Full path : ")
 path=Path(storage_path)
 path.mkdir(parents=True,exist_ok=True)
 log.success("Done")

 log.info("Required informations for PGP Keys")
 name = input(chalk.yellow.bold("Name :"))
 comment = input(chalk.yellow.bold("Comment :"))
 email =  input(chalk.yellow.bold("Email :"))

 if name == "":
  name="Tibis"
 if comment =="":
  comment="Tibis Key"
 if email == "":
  email="tibis@localhost.org"

 default_config=static.default_config
 default_config['pgp_infos']['name']=name
 default_config['pgp_infos']['email']=email
 default_config['pgp_infos']['comment']=comment
 default_config['storage_path']=storage_path

 with open(static.tibis_full_config_location,'w') as file:
  yaml.dump(default_config,file)
  log.success("Ready to go !")

if __name__ == '__main__':
    firstTime()


import sqlite3
import tibis.lib.logger as log
import tibis.lib.static as static

def list():
 try:
  db=static.tibis_db_location
  conn=sqlite3.connect(db)
  cursor_obj = conn.cursor()
  sql = static.tibis_database_list_all
  cursor_obj.execute(sql)
  rows = cursor_obj.fetchall()
  for row in rows:
    name=row[0]
    status=row[3]
    mount_point=row[4]
    if(status=='unlocked'):
      log.unlocked(name + " " +mount_point)
    else:
      log.locked(name)
 except Exception as e:
  log.error(e)
 finally:
  conn.close()

if __name__ == '__main__':
  list()


import sys
import argparse
from tibis.lib.create import create
from tibis.lib.lock import lock
from tibis.lib.unlock import unlock
from tibis.lib.list import list
from tibis.lib.delete import delete
from tibis.lib.static import version,codename
import tibis.lib.logger as log

def args_parser():

	parser = argparse.ArgumentParser(description=f"""
		   _   _   _   _   _  
		  / \ / \ / \ / \ / \ 
		 ( t | i | b | i | s )
		  \_/ \_/ \_/ \_/ \_/	

	A cool tool to manage encrypted directory
    Version : {version} Codename : {codename}""",
    formatter_class=argparse.RawTextHelpFormatter)

	parser.add_argument('action',nargs=1,choices=['create','list' ,'lock', 'unlock','delete'],help="Main action to choose")
	parser.add_argument('name',nargs='?',help="directory name or path required by create, lock, unlock and delete")
	parser.add_argument('destination',nargs='?',help="directory destination required by unlock")

	try:
		args = parser.parse_args(args=None if sys.argv[1:] else ['--help'])
	except SystemExit:
		#Maybe do something here ?
		exit(-1)

	action=args.action[0]
	if(action=='list'):
		list()
		sys.exit(0)
	#Dirname is required here
	if(not args.name):
		log.error("name is required")
		sys.exit(0)
	
	dirname=args.name

	if(action=='create'):
		create(dirname)
		sys.exit(1)

	if(action=='lock'):
		lock(dirname)
		sys.exit(1)

	if(action=='delete'):
		delete(dirname)
		sys.exit(1)

	if(not args.destination):
		log.error("destination is required")
		sys.exit(0)
	destination=args.destination
	if(action=='unlock'):
		unlock(dirname,destination)
		sys.exit(1)